/*
* Copyright 2011 Research In Motion Limited.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package webworks.media.barcode.UI;

import java.util.Hashtable;
import java.lang.StringBuffer;
import javax.microedition.media.MediaException;

import webworks.media.barcode.ErrorObject;

import net.rim.device.api.barcodelib.BarcodeDecoder;
//import net.rim.device.api.barcodelib.BarcodeDecoderListener;
import net.rim.device.api.amms.control.camera.ImageDecoderListener;
import com.google.zxing.Result;
import net.rim.device.api.barcodelib.BarcodeScanner;
import net.rim.device.api.script.ScriptableFunction;
import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.container.MainScreen;

public class ScannerScreen extends MainScreen {

	private BarcodeScanner _scanner;
	
	private ScriptableFunction _captureCallback;
	private ScriptableFunction _errorCallback;
	
	
    
    /**
     * Creates a new ViewFinderScreen object
     */
    public ScannerScreen(final ScriptableFunction captureCallback,final ScriptableFunction errorCallback, final Hashtable hints )
    {
    	_captureCallback = captureCallback;
    	_errorCallback = errorCallback;
    	
    	
        // Initialize the BarcodeDecoder
        BarcodeDecoder decoder = new BarcodeDecoder(hints);
        
        // Create a custom instance of a BarcodeDecoderListener to pop the
        // screen and display results when a  barcode is recognized.
        //BarcodeDecoderListener decoderListener = new BarcodeDecoderListener()
        ImageDecoderListener decoderListener = new ImageDecoderListener()
        {
            /**
             * @see BarcodeDecoderListener#barcodeDecoded(String)
             */
            //public void barcodeDecoded(String rawText)
						 public void imageDecoded(Object data)
            {
             	try
             	{
								  _scanner.getPlayer().close();
							}catch (Exception ex){
            			
            			returnError("Stop Scanner Error: " + ex.getMessage());
							
							}
            
		           Result convertedObject = (Result) data;
		           String barcodeBytes = convertedObject.getText();
				 
		          byte[] stringByte = barcodeBytes.getBytes();
		          String strB = "";
		          String strC = "";
		         
		          for (int k = 0; k < stringByte.length; k++)
		          {
		          	strB += byteToHex(stringByte[k]);
		          	
		          }
		          strC = convertHexToString(strB);
		          
		          returnSuccess(strC);
		        }
        };
            
        try
        {
            // Initialize the BarcodeScanner object and add the associated
            // view finder.                
            _scanner = new BarcodeScanner(decoder, decoderListener);
            _scanner.getVideoControl().setDisplayFullScreen(true);
            add(_scanner.getViewfinder());
            
        }
        catch(Exception e)
        {
            returnError("Error: " + e.getMessage());
        }
                
    }        

    
    /**
     * Informs the BarcodeScanner that it should begin scanning for QR Codes
     */
    public void startScan()
    {
        try
        {
            _scanner.startScan();
        }
        catch(MediaException me)
        {
        	returnError("Error: " + me.getMessage());
        }
    }
    
    
    /**
     * @see net.rim.device.api.ui.Screen#keyChar()
     */
    protected boolean keyChar(char key, int status, int time)
    {
        if (key == Characters.ESCAPE) 
        {
            // Manually stop the scanning process and pop the screen
            try
            {
                _scanner.stopScan();

                _scanner.getPlayer().close();
                
                UiApplication.getUiApplication().popScreen(ScannerScreen.this);
            }
            catch (MediaException me)
            {
            	returnError("Error: " + me.getMessage());
            } 
             catch (Exception ex)
            {
            	returnError("Error: " + ex.getMessage());
            }
        }
        
        return super.keyChar(key, status, time);
    }
    
    
    /**
     * @see net.rim.device.api.ui.Screen#close()
     */
    public void close()
    {
        try 
        {
            _scanner.stopScan();
             	
            _scanner.getPlayer().close();
            
            UiApplication.getUiApplication().popScreen(ScannerScreen.this);
        }
        catch (MediaException me)
        {
        	returnError("Error: " + me.getMessage());
        }
        catch (Exception ex)
        {
        	returnError("Error: " + ex.getMessage());
        }
        
        super.close();
    }
    
    
    /**
     * Pops the ViewFinderScreen and displays text on the main screen
     * 
     * @param text Text to display on the screen
     */
    private void returnSuccess(final String text)
    {
        UiApplication.getUiApplication().invokeLater(new Runnable()
        {
            public void run()
            {                
                UiApplication.getUiApplication().popScreen(ScannerScreen.this);
                try {
                	                	
                	_captureCallback.invoke(null, new Object[] { text });
                	
				} catch (Exception e) {

										returnError("FUNCTION ReturnSuccess failed " + e.getMessage());
					
				}
            } 
        });
    }
    
    /**
     * Pops the ViewFinderScreen and displays text on the main screen
     * 
     * @param text Text to display on the screen
     */
    public void returnError(final String text)
    {
        UiApplication.getUiApplication().invokeLater(new Runnable()
        {
            public void run()
            {                
                 UiApplication.getUiApplication().popScreen(ScannerScreen.this);
                try {
					_errorCallback.invoke(null,new Object[] {new ErrorObject(-1, text)});
				} catch (Exception e) {
				}
            } 
        });
    }
    
    
    public String byteToHex(byte b) {
      // Returns hex String representation of byte b
      char hexDigit[] = {
         '0', '1', '2', '3', '4', '5', '6', '7',
         '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
      };
      char[] array = { hexDigit[(b >> 4) & 0x0f], hexDigit[b & 0x0f] };
      return new String(array);
   }
   
   public String charToHex(char c) {
      // Returns hex String representation of char c
      byte hi = (byte) (c >>> 8);
      byte lo = (byte) (c & 0xff);
      return byteToHex(hi) + byteToHex(lo);
   }
   
    public String convertHexToString(String hex){
 
	  StringBuffer sb = new StringBuffer();
 
	  //49204c6f7665204a617661 split into two characters 49, 20, 4c...
	  for( int i=0; i<hex.length()-1; i+=2 ){
 
	      //grab the hex in pairs
	      String output = hex.substring(i, (i + 2));
	      if (output.equals("1e")){
	      	sb.append("~RS");
	      } else if (output.equals("1d")){
	      	sb.append("~GS");
	      } else if (output.equals("1f")){
	      	sb.append("~US");
	      } else if (output.equals("04")){
	      	sb.append("~EOT");
	      } else {
	      	//convert hex to decimal
	     		int decimal = Integer.parseInt(output, 16);
	      	//convert the decimal to character
	      	sb.append((char)(decimal));
 				}

	  }
 
	  return sb.toString();
  }
  
  
  public static String toHexString(byte bytes[]) {
    if (bytes == null) {
        return null;
    }

    StringBuffer sb = new StringBuffer();
    for (int iter = 0; iter < bytes.length; iter++) {
        byte high = (byte) ( (bytes[iter] & 0xf0) >> 4);
        byte low =  (byte)   (bytes[iter] & 0x0f);
        sb.append(nibble2char(high));
        sb.append(nibble2char(low));
    }

    return sb.toString();
}

private static char nibble2char(byte b) {
    byte nibble = (byte) (b & 0x0f);
    if (nibble < 10) {
        return (char) ('0' + nibble);
    }
    return (char) ('a' + nibble - 10);
}

}
